#!/usr/bin/env python
# encoding: iso-8859-15
#
#Min. Mundo Interactivo-Narrativo en Espaol [Tierra-Media]
#Copyright (C) 2002  Pablo Ruiz Mzquiz
#
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

#
#Fichero: ContenedorObjeto.py

"""Interfaz ContenedorObjeto."""


class ContenedorObjeto(object):
    """Interfaz que debe implementar cualquier entidad que pueda contener un Objeto.
    
    Contiene los mensajes de respuesta a los eventos que produce el objeto.
    
    En la mayora de los lenguajes orientados a objetos hay que crear una clase o
    un interfaz para esto. En Python, debido a ser dinmicamente tipado, esta
    clase en realidad es totalmente innecesaria. La pongo slo como documentacin.
    
    Todos los mtodos tienen que estar redefinidos en subclases.
    """
                        
                        
    def objeto_desaparece(self, objeto):
        """Metodo llamado cuando un objeto desaparece de su contenedor (bien porque ha
	sido destruido, o movido automticamente a otro sitio).

	Se espera que el contenedor, en este mtodo, llame a objeto.liberar().
        """
        pass
                        
                        
    def objeto_aparece(self, objeto):
        """Metodo llamado cuando un objeto aparece en este contenedor (porque
	se ha movido a este lugar).

	Se espera que el contenedor, en este mtodo, llame a objeto.contener().
        """
        pass
                         
                        
    def objeto_reemplazado(self, objeto_antiguo, objeto_nuevo):
        """Metodo llamado cuando un objeto contenido en este contenedor es reemplazado
	por otro distinto.

	Se espera que el contenedor, en este mtodo, llame a objeto_antiguo.liberar()
	y a objeto_nuevo.contener().
        """
        pass
                       


# Cdigo para pruebas unitarias del mdulo.

if (__name__ == '__main__'):
    print "No hay cdigo para pruebas unitarias del mdulo, ya que es un interfaz"
    print "y no tiene cdigo."
