#!/usr/bin/env python
# encoding: iso-8859-15
#
#Min. Mundo Interactivo-Narrativo en Espaol [Tierra-Media]
#Copyright (C) 2002  Pablo Ruiz Mzquiz
#
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

#
#Fichero: Controlador.py

"""Interfaz Controlador."""


class Controlador(object):
    """Interfaz que debe implementar cualquier entidad que controla a un personaje.
    
    Contiene los mensajes de respuesta a los sucesos que producen los personajes.
    
    En la mayora de los lenguajes orientados a objetos hay que crear una clase o
    un interfaz para esto. En Python, debido a ser dinmicamente tipado, esta
    clase en realidad es totalmente innecesaria. La pongo slo como documentacin.
    
    Todos los mtodos tienen que estar redefinidos en subclases.
    """
                        
                        
    def entra_personaje(self, personaje):
        """Metodo llamado cuando otro personaje entra en la misma sala.
        
        personaje -- el personaje que entra.
        """
        pass
                        
                        
    def sale_personaje(self, personaje, nombre_sala_destino):
        """Metodo llamado cuando otro personaje sale de la misma sala.
        
        personaje -- el personaje que sale.
        nombre_sala_destino -- el nombre de la sala a donde va
        """
        pass
                        

    def habla_personaje(self, personaje, mensaje):
        """Metodo llamado cuando otro personaje dice algo en la sala.
        
        personaje -- el personaje que habla.
        mensaje -- lo que dice.
        """
        pass


    def susurra_personaje(self, personaje, mensaje):
        """Metodo llamado cuando otro personaje dice algo slo para ste.
        
        personaje -- el personaje que susurra.
        mensaje -- lo que dice.
        """
	pass    
    
                        
    def grita_personaje(self, personaje, mensaje):
        """Metodo llamado cuando otro personaje dice algo que oyen *todos*
	los personajes de Min (esto habra que cambiarlo porque es OOC).
        
        personaje -- el personaje que grita.
        mensaje -- lo que dice.
        """
        pass
                        
                        
    def actua_personaje(self, personaje, accion):
        """Metodo llamado cuando otro personaje en la sala describe lo
	que hace de forma novelada.
        
        personaje -- el personaje que actua.
        accion -- lo que hace (el texto est expresado para colocarle
	          el nombre del personaje al principio, por ejemplo
		  "coge una lampara" -> "Hirunatan coge una lampara").
        """
        pass


    def actua_individual_personaje(self, personaje, accion):
        """Metodo llamado cuando otro personaje describe lo que hace
	de forma que slo lo percibe ste.
        
        personaje -- el personaje que acta.
        accion -- lo que hace (el texto est expresado para colocarle
	          el personaje al principio, y est en segunda persona,
		  por ejemplo "te da una espada" -> "Aranarth te da una
		  espada").
        """
	pass    


    def actua_uno_mismo(self, accion):
	"""Metodo llamado cuando el personaje se informa a si mismo (a su
	controlador, en realidad) de algo que hace.

	Normalmente esto sucede de forma asncrona, es decir, no como
	respuesta directa a una accin del jugador, sino indirectamente.

	accion -- lo que hace (el texto va en segunda persona, y se
	          mostrar tal cual en pantalla, por ejemplo "No puedes
		  seguir a Hirunatan porque la sala est llena").
	"""
	pass
                        
                        
    def actua_global_personaje(self, personaje, accion):
        """Metodo llamado cuando otro personaje describe de forma novelada
	algo que hace y que ven *todos* los personajes de Min.
        
        personaje -- el personaje que actua.
        accion -- lo que hace (el texto est expresado para colocarle
	          el nombre del personaje al principio, por ejemplo
		  "ha entrado en Min" -> "Hirunatan ha entrado en Min").
        """
        pass


    def suceso_entorno(self, suceso):
	"""Mtodo llamado cuando ocurre un fenmeno ambiental, independiente
	de ningn personaje concreto.
	
	personaje -- el personaje que acta.
	suceso -- lo que ocurre (el texto est expresado de forma impersonal,
	          y se mostrar tal cual en pantalla, por ejemplo "De pronto
		  se hace un silencio sepulcral en la sala").
	"""
	pass
        
        
    def ataca_personaje(self, personaje, result):
        """Metodo llamado cuando otro personaje inicia un ataque a este.
        
        personaje -- el personaje que ha atacado.
        result -- el resultado del combate (Personaje.ATAQUE_EXITO, etc.)
        """
        pass


    def movimiento_automatico(self):
	"""Metodo llamado cuando el personaje se ha movido automaticamente
	a otra sala, por estar siguiendo a otro personaje.
	"""
	pass


    def paga_personaje(self, personaje, monedas):
	"""Metodo llamado cuando otro personaje paga dinero a este.

	personaje -- el personaje que ha pagado
	monedas -- el numero de monedas que nos ha dado
	"""
	pass



# Cdigo para pruebas unitarias del mdulo.

if (__name__ == '__main__'):
    print "No hay cdigo para pruebas unitarias del mdulo, ya que es un interfaz"
    print "y no tiene cdigo."
